# group-array [![NPM version](https://img.shields.io/npm/v/group-array.svg?style=flat)](https://www.npmjs.com/package/group-array) [![NPM downloads](https://img.shields.io/npm/dm/group-array.svg?style=flat)](https://npmjs.org/package/group-array) [![Build Status](https://img.shields.io/travis/doowb/group-array.svg?style=flat)](https://travis-ci.org/doowb/group-array)

Group array of objects into lists.

## Table of Contents

- [Install](#install)
- [Usage](#usage)
- [Examples](#examples)
- [About](#about)
  * [Related projects](#related-projects)
  * [Contributing](#contributing)
  * [Building docs](#building-docs)
  * [Running tests](#running-tests)
  * [Author](#author)
  * [License](#license)

_(TOC generated by [verb](https://github.com/verbose/verb) using [markdown-toc](https://github.com/jonschlinkert/markdown-toc))_

## Install

Install with [npm](https://www.npmjs.com/):

```sh
$ npm install --save group-array
```

## Usage

```js
var groupArray = require('group-array');
```

## Examples

```js
var arr = [
  {tag: 'one', content: 'A'},
  {tag: 'one', content: 'B'},
  {tag: 'two', content: 'C'},
  {tag: 'two', content: 'D'},
  {tag: 'three', content: 'E'},
  {tag: 'three', content: 'F'}
];

// group by the `tag` property
groupArray(arr, 'tag');
```

**results in:**

```js
{
  one: [
    {tag: 'one', content: 'A'},
    {tag: 'one', content: 'B'}
  ],
  two: [
    {tag: 'two', content: 'C'},
    {tag: 'two', content: 'D'}
  ],
  three: [
    {tag: 'three', content: 'E'},
    {tag: 'three', content: 'F'}
  ]
}
```

**Group by multiple, deeply nested properties**

```js
// given an array of object, like blog posts...
var arr = [
  { data: { year: '2014', tag: 'one', month: 'jan', day: '01'}, content: '...'},
  { data: { year: '2014', tag: 'one', month: 'jan', day: '01'}, content: '...'},
  { data: { year: '2014', tag: 'one', month: 'jan', day: '02'}, content: '...'},
  { data: { year: '2014', tag: 'one', month: 'jan', day: '02'}, content: '...'},
  { data: { year: '2014', tag: 'one', month: 'feb', day: '10'}, content: '...'},
  { data: { year: '2014', tag: 'one', month: 'feb', day: '10'}, content: '...'},
  { data: { year: '2014', tag: 'one', month: 'feb', day: '12'}, content: '...'},
  { data: { year: '2014', tag: 'one', month: 'feb', day: '12'}, content: '...'},
  { data: { year: '2014', tag: 'two', month: 'jan', day: '14'}, content: '...'},
  { data: { year: '2014', tag: 'two', month: 'jan', day: '14'}, content: '...'},
  { data: { year: '2014', tag: 'two', month: 'jan', day: '16'}, content: '...'},
  { data: { year: '2014', tag: 'two', month: 'jan', day: '16'}, content: '...'},
  { data: { year: '2014', tag: 'two', month: 'feb', day: '18'}, content: '...'},
  { data: { year: '2015', tag: 'two', month: 'feb', day: '18'}, content: '...'},
  { data: { year: '2015', tag: 'two', month: 'feb', day: '10'}, content: '...'},
  { data: { year: '2015', tag: 'two', month: 'feb', day: '10'}, content: '...'},
  { data: { year: '2015', tag: 'three', month: 'jan', day: '01'}, content: '...'},
  { data: { year: '2015', tag: 'three', month: 'jan', day: '01'}, content: '...'},
  { data: { year: '2015', tag: 'three', month: 'jan', day: '02'}, content: '...'},
  { data: { year: '2015', tag: 'three', month: 'jan', day: '02'}, content: '...'},
  { data: { year: '2015', tag: 'three', month: 'feb', day: '01'}, content: '...'},
  { data: { year: '2015', tag: 'three', month: 'feb', day: '01'}, content: '...'},
  { data: { year: '2015', tag: 'three', month: 'feb', day: '02'}, content: '...'},
  { data: { year: '2015', tag: 'three', month: 'feb', day: '02'}, content: '...'}
]
```

Pass a list or array of properties:

```js
groupArray(arr, 'data.year', 'data.tag', 'data.month', 'data.day');
```

**Results in something like this: (abbreviated)**

```js
{
  '2014':
   { one:
      { jan:
         { '01':
            [ { data: { year: '2014', ... },
              { data: { year: '2014', ... } ],
           '02':
            [ { data: { year: '2014', ... },
              { data: { year: '2014', ... } ] },
        feb:
         { '10':
            [ { data: { year: '2014', ... },
              { data: { year: '2014', ... } ],
           '12':
            [ { data: { year: '2014', ... },
              { data: { year: '2014', ... } ] } },
     two:
      { jan:
         { '14':
            [ { data: { year: '2014', ... },
              { data: { year: '2014', ... } ],
           '16':
            [ { data: { year: '2014', ... },
              { data: { year: '2014', ... } ] },
        feb:
         { '18':
            [ { data: { year: '2014', ... } ] } } },
  '2015':
   { two:
      { feb:
         { '10':
            [ { data: { year: '2015', ... },
              { data: { year: '2015', ... } ],
           '18':
            [ { data: { year: '2015', ... } ] } },
     three:
      { jan:
         { '01':
            [ { data: { year: '2015', ... },
              { data: { year: '2015', ... } ],
           '02':
            [ { data: { year: '2015', ... },
              { data: { year: '2015', ... } ] },
        feb:
         { '01':
            [ { data: { year: '2015', ... },
              { data: { year: '2015', ... } ],
           '02':
            [ { data: { year: '2015', ... },
              { data: { year: '2015', ... } ] } } }};
```

## About

### Related projects

* [arr-reduce](https://www.npmjs.com/package/arr-reduce): Fast array reduce that also loops over sparse elements. | [homepage](https://github.com/jonschlinkert/arr-reduce "Fast array reduce that also loops over sparse elements.")
* [get-value](https://www.npmjs.com/package/get-value): Use property paths (`a.b.c`) to get a nested value from an object. | [homepage](https://github.com/jonschlinkert/get-value "Use property paths (`a.b.c`) to get a nested value from an object.")
* [group-object](https://www.npmjs.com/package/group-object): Group object keys and values into lists. | [homepage](https://github.com/doowb/group-object "Group object keys and values into lists.")
* [union-value](https://www.npmjs.com/package/union-value): Set an array of unique values as the property of an object. Supports setting deeply… [more](https://github.com/jonschlinkert/union-value) | [homepage](https://github.com/jonschlinkert/union-value "Set an array of unique values as the property of an object. Supports setting deeply nested properties using using object-paths/dot notation.")

### Contributing

Pull requests and stars are always welcome. For bugs and feature requests, [please create an issue](../../issues/new).

### Building docs

_(This document was generated by [verb-generate-readme](https://github.com/verbose/verb-generate-readme) (a [verb](https://github.com/verbose/verb) generator), please don't edit the readme directly. Any changes to the readme must be made in [.verb.md](.verb.md).)_

To generate the readme and API documentation with [verb](https://github.com/verbose/verb):

```sh
$ npm install -g verb verb-generate-readme && verb
```

### Running tests

Install dev dependencies:

```sh
$ npm install -d && npm test
```

### Author

**Brian Woodward**

* [github/doowb](https://github.com/doowb)
* [twitter/doowb](http://twitter.com/doowb)

### License

Copyright © 2016, [Brian Woodward](https://github.com/doowb).
Released under the [MIT license](https://github.com/doowb/group-array/blob/master/LICENSE).

***

_This file was generated by [verb](https://github.com/verbose/verb), v0.9.0, on July 15, 2016._